// ConnectedComponent.cpp: implementation of the ConnectedComponent class.
//
//////////////////////////////////////////////////////////////////////

#include "ConnectedComponent.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
NODE_INFO(ConnectedComponent,"Image", "IMAGE_IN", "LIST_OUT", "X_SAMPLING:Y_SAMPLING")

  

ConnectedComponent::ConnectedComponent(string nodeName, ParameterSet params)
: Node(nodeName, params) ,
  MAXIMUM_RECURSION_LEVEL(10000)
 
 
{
  try {
    m_IMAGE_ID = addInput("IMAGE_IN");
    addOutput("LIST_OUT");
    m_image = NULL;
    
    m_xSampling = object_cast<Int>(parameters.get("X_SAMPLING")).val();
    m_ySampling = object_cast<Int>(parameters.get("Y_SAMPLING")).val();
      

  }
  catch (BaseException *e) {
    //Something weird happened
    throw e->add(new NodeException (this,
				    string("error caught in ConnnectedComponent::constructor"), 
				    __FILE__,__LINE__));
  }      
  

}


ConnectedComponent::~ConnectedComponent()
{
  
}

void ConnectedComponent::get_components() {
  
  for (int x = 0; x < m_xSize; x += m_xSampling) {
    for (int y = 0; y < m_ySize; y += m_ySampling) {
      if (m_image->get_val (x,y)) {
	//crer le rectangle
	CRect rect(x,y,x,y);
	
	//remettre  zro le niveau de rcursion
	
	m_rlevel = 0;
	//trouver les composants connects
	get_components_recursive (rect,x,y);
	
	//ajouter ce CRect  la liste...
	
	
	if ((rect.Height() * rect.Width()) > (m_xSampling * m_ySampling)) {
	  /* 
	     ATTENTION IL RESTE A VERIFIER SI LE RECT
	     NE FAIS PAS PARTIE D'UN PLUS GRAND RECT
	     IL FAUDRA FAIRE UNION
	  */	
	  MergeRectangle(rect);	
	}
	/*else {
	  //c'est un point isol
	  delete rect;
	  }*/
      }//if
    }//for y
    //cout<<"rect done"<<endl;
  }//for x

}

void ConnectedComponent::get_components_recursive (CRect &rect, int x,int y) {
  
  
  m_rlevel++;
  
  //ce pixel retourne a zro
  m_image->set_val(x,y,0x0);

  //adding point in the rectangle
  rect.add_point(CPoint(x,y));
  
  
  //agrandir le CRect au besoin
  CPoint ulc = rect.TopLeft();
  CPoint lrc = rect.BottomRight();
  
  //coin gauche haut
  if (x < ulc.x) {
    ulc.x = x;
  }
  if (y >= lrc.y) {
    if (y + m_ySampling - 1 < m_ySize) {
      lrc.y = y + m_ySampling - 1;
    }
    else {
      lrc.y = y;
    }
  }
  
  //coin droit bas
  if (x >= lrc.x) {
    if (x + m_xSampling - 1< m_xSize) {
      lrc.x = x + m_xSampling - 1;
    }
    else {
      lrc.x = x;
    }
  }
  if (y < ulc.y) {
    ulc.y = y;
  }
  
  //mise  jour du rectangle
  rect.SetRect(ulc,lrc);
  
  //vrification pour la rcursion
  if (m_rlevel > MAXIMUM_RECURSION_LEVEL) {
    m_rlevel--;
    return;
  }
  
  //vrification des 8 voisins
  
  //haut
  if ( y + m_ySampling < m_ySize) {
    if (m_image->get_val(x,y+m_ySampling)) {
      get_components_recursive (rect,x,y+m_ySampling);
    }		
  }
  
  //haut droit
  if ( y + m_ySampling < m_ySize && x + m_xSampling < m_xSize) {
    if (m_image->get_val(x + m_xSampling,y+m_ySampling)) {
      get_components_recursive (rect,x+m_xSampling,y+m_ySampling);
    }		
  }
  
  //droit
  if (x + m_xSampling < m_xSize) {
    if (m_image->get_val(x+m_xSampling,y)) {
      get_components_recursive (rect,x+m_xSampling,y);
    }		
  }
  
  //bas droit
  if ( y-m_ySampling > 0 && x+m_xSampling < m_xSize) {
    if (m_image->get_val(x+m_xSampling,y-m_ySampling)) {
      get_components_recursive (rect,x+m_xSampling,y-m_ySampling);
    }		
  }
  
  //bas
  if ( y-m_ySampling > 0) {
    if (m_image->get_val(x,y-m_ySampling)) {
      get_components_recursive (rect,x,y-m_ySampling);
    }		
  }
  
  //bas gauche
  if ( y-m_ySampling > 0 && x-m_xSampling > 0) {
    if (m_image->get_val(x-m_xSampling,y-m_ySampling)) {
      get_components_recursive (rect,x-m_xSampling,y-m_ySampling);
    }		
  }
  
  //gauche
  if (x-m_xSampling > 0) {
    if (m_image->get_val(x-m_xSampling,y)) {
      get_components_recursive (rect,x-m_xSampling,y);
    }		
  }
  
  
  //haut gauche
  if ( y + m_ySampling < m_ySize && x-m_xSampling > 0) {
    if (m_image->get_val(x-m_xSampling,y+m_ySampling)) {
      get_components_recursive (rect,x-m_xSampling,y+m_ySampling);
    }		
  }
  
  
  m_rlevel--;
}



void ConnectedComponent::MergeRectangle(CRect &rect)
{
  /*
    Nous inserons le nouveau rectangle seulement si necessaire
  */
  
  bool merged = false;
  
  list<CRect>::iterator iter;
  
  for (iter = m_RectList->begin(); iter != m_RectList->end(); iter++) {
    
    CRect temp_rect;
    temp_rect.IntersectRect(rect, (*iter));
    if (temp_rect.IsRectEmpty()) {
      continue;
    }
    else {
      temp_rect  = (*iter);
      (*iter).UnionRect(temp_rect,rect);
      merged = true;
      break;
    }
    
  }
  
  if (!merged) {
    m_RectList->push_back(CRect(rect));
  }
  /*else {
    delete rect;
    }*/
}

ObjectRef ConnectedComponent::getOutput (int output_id, int count) {

  if (!hasOutput(output_id)) throw new NodeException (this, "Cannot getOutput id",__FILE__,__LINE__);
  
  if (count != processCount) {
    //We are updating our output only if needed
    
    try {
      
      //getting all data from our inputs.
      int OutputID = inputs[m_IMAGE_ID].outputID;
      ObjectRef im = inputs[m_IMAGE_ID].node->getOutput(OutputID, count);
      
      //getting image data
      GreyScale8Image image_copy(object_cast<GreyScale8Image>(im));
      m_image = &image_copy;
      m_xSize = m_image->get_width();
      m_ySize = m_image->get_height();

      m_RectList = new List<CRect>;
      get_components();
      
      cerr<<"Found "<<m_RectList->size()<<" components "<<endl;



      GreyScale8Image image_temp(object_cast<GreyScale8Image>(im));

      List<CRect>::iterator iter = m_RectList->begin();
      
      for ( int i = 0;
	    i < m_RectList->size();
	   iter++,i++) {
	image_temp.draw_rect(*iter);
	
      }
      
      string fname("components.ppm");
      image_temp.write_ppm(fname);

      m_output = ObjectRef(m_RectList);

    } //end of try block
    catch (BaseException *e) {
      //Something weird happened
      throw e->add(new NodeException (this,
				      string("error caught in ConnnectedComponent::getOutput(int,int)"), 
				      __FILE__,__LINE__));
     }      
     
    //updating processCount
    processCount = count;                 
  }
  
  
  return (m_output);
  
} 
 
