// RGB24Image.cc: implementation of the RGB24Image class.
//
//////////////////////////////////////////////////////////////////////
#include "RGB24Image.h"
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include "errno.h"
#include "Exception.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
const long RGB24Image::RED_MASK = 0xFF000000;
const long RGB24Image::GREEN_MASK = 0x00FF0000;
const long RGB24Image::BLUE_MASK = 0x0000FF00;

RGB24Image::RGB24Image()
: Image(RGB_24)
{

}

RGB24Image::~RGB24Image()
{
  if (m_data) {
    delete [] m_data;
    m_data = NULL;
  }
}

RGB24Image::RGB24Image(int width, int height)
  : Image(RGB_24)
{
  m_width = width;
  m_height = height;
  m_data = new unsigned char[m_width * m_height * 4];
}

RGB24Image::RGB24Image(int width, int height, unsigned char *data)
  : Image(RGB_24)
{
  m_width = width;
  m_height = height;
  m_data = new unsigned char[m_width * m_height * 4];
  memcpy(m_data,data,m_width * m_height * 4);	
  
}

RGB24Image::RGB24Image(const RGB24Image &copy)
  : Image(RGB_24)
{
  m_width = copy.m_width;
  m_height = copy.m_height;
  m_data = new unsigned char [m_width * m_height * 4];
  memcpy(m_data,copy.m_data,m_width*m_height*4);
  m_type = copy.m_type;
}
bool RGB24Image::read_ppm(string &filename) {
  
  unsigned char rgb[3];
  int width;
  int height;
  int fd;
  char start_code[2];
  char compare_code[2] = {'P','6'};
  unsigned char *p;
  char *ptr;
  char buf[256];
  char num[32];

  if ((fd = open (filename.c_str(), O_RDONLY, 0644)) < 0) {
    printf ("ppm open failed (reading): %d\n", errno);
    return false;
  }

  read (fd, &start_code[0],2);
  
  
  if (strcmp(start_code,compare_code) > 0) {
    cerr<<"Invalid start code : "<<start_code<<endl;
    close(fd);
    return false;
  }

  if (m_data != NULL) {
    cout<<"del m_data"<<endl;
    delete [] m_data;
  }
  
  ptr = &buf[0];
  int cr_count = 0;
  int size = 0;

  while (cr_count != 3) {
    read(fd,ptr,1);
    
    if (*ptr == '\n') {
      cr_count++;
    }  
    if (*ptr == '#') {
      while (1) {
	read(fd,ptr,1);
	if (*ptr == '\n'){
	  ptr--;
	  break;
	}
      }
    }
    size++;
    ptr++;    
  }  

  buf[size] = '\0';

  
  //reading the header
  
  strcpy(num,strtok (buf,"\n "));
  width = atoi(num);

  strcpy(num,strtok (NULL,"\n "));
  height = atoi(num);

  

  printf("creating image %i X %i \n",width,height);

  m_width = width;
  m_height = height;
  

  
  //memory allocaltion
  m_data = new unsigned char[4 * width * height];
  p = & m_data[0];

  for (int i = 0; i < height; i++) {
    for (int j = 0; j < width; j++) {
      if (read(fd, &rgb[0], 3) != 3) {
	delete [] m_data;
	m_width = 0;
	m_height = 0;
	close(fd);
	return false;
      }
      
      //*p = 0x0;
 
      *p = rgb[0];p++;		/* red */
      *p = rgb[1];p++;		/* green */
      *p = rgb[2];p++;	        /* blue */
      p++;		/* NULL byte */
    }
  }

  

  close(fd);

  return true;
}

bool RGB24Image::write_ppm(string &filename) {

  unsigned char rgb[3];
  unsigned char *p;
  char header[256];
  int fd;


  sprintf (header, "P6\n%d %d\n255\n", m_width, m_height);

  if ((fd = open (filename.c_str(), O_WRONLY | O_CREAT, 0644)) < 0) {
    printf ("ppm open failed (writing): %d\n", errno);
    return false;
  }
  
  write (fd, &header[0], strlen(header));

  p = &m_data[0];
  for (int j=0; j<m_height; j++) {
    for (int k=0; k<m_width; k++) {

      rgb[0] = *p++;		/* red*/
      rgb[1] = *p++;		/* green */
      rgb[2] = *p++;		/* blue */
      p++;			/* NULL byte */
      write(fd, &rgb[0], 3);	/* not very efficient */
    }
  }

  close (fd);
  return true;
}
void RGB24Image::draw_rect(CRect &rect, unsigned char val) {

  CPoint ulc = rect.TopLeft();
  CPoint lrc = rect.BottomRight();

  //boundary check
  if (ulc.x < 0 ||
      ulc.y < 0 ||
      ulc.x >= m_width ||
      ulc.y >= m_height) {
    throw new GeneralException("CRect (ulc) mismatch",__FILE__,__LINE__);
  }

  //boundary check
  if (lrc.x < 0 ||
      lrc.y < 0 ||
      lrc.x >= m_width ||
      lrc.y >= m_height) {
    throw new GeneralException("CRect (lrc) mismatch",__FILE__,__LINE__);
  }

  unsigned char *p = NULL;
  //drawing vertical
  for (int i = ulc.y; i <= lrc.y;i++) {
    p = &m_data[(i * m_width + ulc.x) * 4];
    //left
    *p++ = val; //RED
    *p++ = val; //GREEN
    *p++ = val; //BLUE
    //right
    p = &m_data[(i * m_width + lrc.x) * 4];
    *p++ = val; //RED
    *p++ = val; //GREEN
    *p++ = val; //BLUE
  }
  //drawing horizontal
  for (int j = lrc.x; j <= lrc.x; j++) {
    p = &m_data[(ulc.y * m_width + j) * 4];
    //top
    *p++ = val; //RED
    *p++ = val; //GREEN
    *p++ = val; //BLUE
    //bottom
    p = &m_data[(lrc.x * m_width + j) * 4];
    *p++ = val; //RED
    *p++ = val; //GREEN
    *p++ = val; //BLUE
  }
}


#ifdef RGB24TEST
int main (int argc, char* argv[]) {
  
  if (argc != 2) {
    cerr<<"Required args"<<endl;
    return -1;
  }
  string name(argv[1]);
  cout<<"Creating image"<<endl;
  RGB24Image im;
  cout<<"reading ppm"<<endl;
  im.read_ppm(name);
  string outname("output.ppm");
  im.write_ppm(outname);

  return 0;
}
#endif
