// Copyright (C) 2000 Dominic Letourneau (doumdi@yahoo.com)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "Threshold.h"
#include "Object.h"
#include "ObjectRef.h"
#include "Exception.h"
#include "Kernel.h"
#include <math.h>

using namespace std;

NODE_INFO(Threshold,"Image", "IMAGE1_IN:THRESHOLD_IN", "IMAGE_OUT", "")

Threshold::Threshold(string nodeName, ParameterSet params)
: Node(nodeName, params)
{
  // Add the node's input
  m_IMAGE1_ID = addInput("IMAGE1_IN");
  m_THRESHOLD_ID = addInput("THRESHOLD_IN");
  
  // Add the node's output
  addOutput("IMAGE_OUT");
  
}

Threshold::~Threshold()
{
  return;
}
ObjectRef Threshold::getOutput (int output_id, int count) {

  if (!hasOutput(output_id)) throw new NodeException (this, "Cannot getOutput id",__FILE__,__LINE__);
  
  if (count != processCount) {
    //We are updating our output only if needed
    
    try {
      
      //getting all data from our inputs.
      int OutputID = inputs[m_IMAGE1_ID].outputID;
      GreyScale8Image &im1 = object_cast<GreyScale8Image>(inputs[m_IMAGE1_ID].node->getOutput(OutputID, count));
      
      OutputID = inputs[m_THRESHOLD_ID].outputID;
      int tsh = object_cast<Int>(inputs[m_THRESHOLD_ID].node->getOutput(OutputID,count)).val();

      cout<<"Got threshold :  "<<tsh<<endl;
 
      int width = im1.get_width();
      int height = im1.get_height();
      
      GreyScale8Image *im_ptr = new GreyScale8Image(width,height);
      
      unsigned char *im1_ptr = im1.get_data();
      unsigned char *out_ptr = im_ptr->get_data();

      for (int i = 0; i < height * width; i++) {
	if (*im1_ptr++ >= tsh) {
	  *out_ptr++ = 255;
	}
	else {
	  *out_ptr++ = 0;
	}
      }



      m_output = ObjectRef(im_ptr);

    } //end of try block
    catch (BaseException *e) {
      //Something weird happened
      //e->print();
      throw e->add(new NodeException (this,
				      string("error caught in Threshold::getOutput(int,int)"), 
				      __FILE__,__LINE__));
     }      
     
    //updating processCount
    processCount = count;                 
  }
  
  
  return (m_output);
  
} 
 
 
